/* 
 * File:   main.c
 * Author: Stijn
 *
 * Created on 21/06/2016 21:40
 */

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include "global.h"

// CONFIG1
#pragma config FEXTOSC = OFF    // FEXTOSC External Oscillator mode Selection bits (Oscillator not enabled)
#pragma config RSTOSC = HFINT32 // Power-up default value for COSC bits (HFINTOSC with 2x PLL (32MHz))
#pragma config CLKOUTEN = OFF   // Clock Out Enable bit (CLKOUT function is disabled; I/O or oscillator function on OSC2)
#pragma config CSWEN = ON       // Clock Switch Enable bit (Writing to NOSC and NDIV is allowed)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is enabled)

// CONFIG2
#pragma config MCLRE = OFF      // Master Clear Enable bit (MCLR/VPP pin function is digital input; MCLR internally disabled; Weak pull-up under control of port pin's WPU control bit.)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config WDTE = OFF       // Watchdog Timer Enable bits (WDT disabled; SWDTEN is ignored)
#pragma config LPBOREN = OFF    // Low-power BOR enable bit (ULPBOR disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable bits (Brown-out Reset disabled)
#pragma config BORV = LOW       // Brown-out Reset Voltage selection bit (Brown-out voltage (Vbor) set to 2.45V)
#pragma config PPS1WAY = OFF    // PPSLOCK bit One-Way Set Enable bit (The PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence))
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable bit (Stack Overflow or Underflow will cause a Reset)
#pragma config DEBUG = OFF      // Debugger enable bit (Background debugger disabled)

// CONFIG3
#pragma config WRT = OFF        // User NVM self-write protection bits (Write protection off)
#pragma config LVP = OFF        // Low Voltage Programming Enable bit (High Voltage on MCLR/VPP must be used for programming.)

// CONFIG4
#pragma config CP = OFF         // User NVM Program Memory Code Protection bit (User NVM code protection disabled)
#pragma config CPD = OFF        // Data NVM Memory Code Protection bit (Data NVM code protection disabled)


#define Sensor      PORTAbits.RA0
#define LED         LATAbits.LATA1
#define Output      LATAbits.LATA2

const int MaxValue = 1023;

void main() {
  
    //Init       76543210   
    TRISA    = 0b11111000;
    TRISB    = 0b11000000;
    TRISC    = 0b11111001; 
    LATB     = 0b00000000;
    LATC     = 0b00000000;
    
    ANSELA   = 0b00000000;
    ANSELB   = 0b11000000;
    ANSELC   = 0b10000001;
    INLVLCbits.INLVLC3 = 1;
    INLVLCbits.INLVLC6 = 1;
    
    Adcinit();
            
    //*************************************************************************
    //PWM
    
    /*Use the desired output pin RxyPPS control to
    select CCPx as the source and disable the
    CCPx pin output driver by setting the associated
    TRIS bit.*/                  
    RC4PPS = 0b01100; //Red
    RC5PPS = 0b01101; //Green
    RA4PPS = 0b01110; //Blue   
    RA5PPS = 0b01111; //White   
    
    //Load the PR2 register with the PWM period value.
    PR2 = 0xFF;
    
    /*Configure the CCP module for the PWM mode
    by loading the CCPxCON register with the
    appropriate values.*/
    CCP1CONbits.CCP1FMT     = 0;
    CCP1CONbits.CCP1MODE    = 0b1111; 
    CCPTMRSbits.C1TSEL      = 0b01; 
    CCP1CONbits.CCP1EN      = 1;
    
    CCP2CONbits.CCP2FMT     = 0;
    CCP2CONbits.CCP2MODE    = 0b1111;
    CCPTMRSbits.C2TSEL      = 0b01;
    CCP2CONbits.CCP2EN      = 1;
    
    CCP3CONbits.CCP3FMT     = 0;
    CCP3CONbits.CCP3MODE    = 0b1111;
    CCPTMRSbits.C3TSEL      = 0b01;     
    CCP3CONbits.CCP3EN      = 1;
    
    CCP4CONbits.CCP4FMT     = 0;
    CCP4CONbits.CCP4MODE    = 0b1111;
    CCPTMRSbits.C4TSEL      = 0b01;     
    CCP4CONbits.CCP4EN      = 1;
    
    /*Load the CCPRxL register and the CCPRxH
    register bits, with the PWM duty cycle value and
    configure the CCPxFMT bit of the CCPxCON
    register to set the proper register alignment.*/
    SetRed(0);   
    SetGreen(0);
    SetBlue (0);
    SetWhite (0);
    
    /*Configure and start Timer2, 4 or 6.
    ? Clear the TMR2/4/6IF interrupt flag bits of
    the PIR4 register. See Note below.*/
    PIR1bits.TMR2IF = 0;
    
    /*Configure the T2/4/6CKPS bits of the
    T2/4/6CON register with the Timer
    prescale value.*/
    T2CONbits.T2CKPS  = 0b11; 
    
    /*Enable the Timer by setting the
    TMR2/4/6ON bit of the T2/4/6CON
    register.*/
    T2CONbits.TMR2ON  = 1;
    
    /*Enable PWM output pin:
    Enable the CCPx pin output driver by
    clearing the associated TRIS bit.*/
    TRISCbits.TRISC4        = 0;
    TRISCbits.TRISC5        = 0;
    TRISAbits.TRISA4        = 0;
    TRISAbits.TRISA5        = 0;
    
    
       
    while (1){
        if ((PORTCbits.RC3 == 1) && (PORTCbits.RC6 == 1)){
            //Uit
            SetRed(0);
            SetGreen(0);
            SetBlue(0);
            SetWhite(0);
        }   
        if ((PORTCbits.RC3 == 0) && (PORTCbits.RC6 == 1)){
            //RGB Pot
            int Level, teller;
            int SetRd, SetGr, SetBl, SetWt;
            Level      = AnaIn(0b001110); //ANB6
            Level = Level >> 1;
            Level = Level * 2;
            
            SetRd = 0;
            SetGr = 0;
            SetBl = 0;
            SetWt = 0;

            if (Level < 100){
                //groen+
                teller = Level * 10;                
                SetRd = MaxValue - LookUp(teller); 
                SetGr = LookUp(teller);               
            }else if (Level < 200){
                //Rood-
                teller = (Level - 100) * 10;
                SetRd = MaxValue - LookUp(teller); 
                SetGr = LookUp(teller);      
            }else if (Level < 300){
                //Blauw+
                teller = (Level - 200) * 10;
                SetGr = MaxValue - LookUp(teller); 
                SetBl = LookUp(teller);                 
            }else if (Level < 400){
                //groen-
                teller = (Level - 300) * 10;
                SetBl = (MaxValue / 2) + LookUp(teller); 
                SetGr = (MaxValue / 2) - LookUp(teller); 
            }else if (Level < 500){
                //Rood+
                teller = (Level - 400) * 10;
                SetBl = MaxValue - LookUp(teller); 
                SetRd = LookUp(teller); 
            }else if (Level < 600){
                //Blauw-
                teller = (Level - 500) * 10;
                SetRd = (MaxValue / 2) + LookUp(teller); 
                SetBl = (MaxValue / 2) - LookUp(teller); 
            }else{
               SetRd = MaxValue;
               SetGr = 0;
               SetBl = 0;
               SetWt = 0;
            }     
             
            CheckLight();
            SetRed  (SetRd);
            SetGreen(SetGr);
            SetBlue (SetBl);  
            SetWhite(SetWt); 
        }   
        if ((PORTCbits.RC3 == 1) && (PORTCbits.RC6 == 0)){
            //Wit
            CheckLight();
            SetRed(0);
            SetGreen(0);
            SetBlue(0);
            SetWhite(MaxValue);
//            while ((PORTCbits.RC3 == 1) && (PORTCbits.RC6 == 0)){ 
//                CheckLight();          
//            }
        }     
        if ((PORTCbits.RC3 == 0) && (PORTCbits.RC6 == 0)){
            //RGB Fade
            SetRed(0);
            SetGreen(0);
            SetBlue(0);
            SetWhite(0);
            RGB_Fade();
        }        
    }     
}

void RGB_Fade(){
    char Modus = 0;
    int SetRd, SetGr, SetBl = 0;
    int teller = 0;
    int LichtSterkte;
    int Snelheid;
    
    while((PORTCbits.RC3 == 0) && (PORTCbits.RC6 == 0)){
       
        Snelheid  = AnaIn(0b001111); //ANB7 
        switch (Modus){
            case 0:
                //groen+
                teller ++;
                SetRd = MaxValue - LookUp(teller); 
                SetGr = LookUp(teller); 
                if (teller == MaxValue){
                    teller = 0;
                    Modus ++;
                }                    
                break;
            case 1:  
                //Rood-
                teller ++;
                SetGr = (MaxValue / 2) + LookUp(teller); 
                SetRd = (MaxValue / 2) - LookUp(teller); 
                if (teller == MaxValue){
                    teller = 0;
                    Modus ++;
                }                    
                break;
            case 2:
                //Blauw+
                teller ++;
                SetGr = MaxValue - LookUp(teller); 
                SetBl = LookUp(teller); 
                if (teller == MaxValue){
                    teller = 0;
                    Modus ++;
                }                              
                break;
            case 3: 
                //groen-
                teller ++;
                SetBl = (MaxValue / 2) + LookUp(teller); 
                SetGr = (MaxValue / 2) - LookUp(teller); 
                if (teller == MaxValue){
                    teller = 0;
                    Modus ++;
                }                  
                break;
            case 4:
                //Rood+
                teller ++;
                SetBl = MaxValue - LookUp(teller); 
                SetRd = LookUp(teller); 
                if (teller == MaxValue){
                    teller = 0;
                    Modus ++;
                }                
                break;
            case 5: 
                //Blauw-
                teller ++;
                SetRd = (MaxValue / 2) + LookUp(teller); 
                SetBl = (MaxValue / 2) - LookUp(teller); 
                if (teller == MaxValue){
                    teller = 0;
                    Modus ++;
                }                  
                break;                  
            default:
                Modus = 0;  
        }
        CheckLight();
        SetRed(SetRd);
        SetGreen(SetGr);
        SetBlue(SetBl);
        for(int i =  0; i < (Snelheid / 2); i++ ){
            __delay_ms(1); 
        }
   }   
}

void SetRed(int value){
    if (value > 1023){
        value = 1023;
    }
    CCPR1L = value & 0x00FF;
    CCPR1H = value >> 8; 
}
void SetGreen(int value){
    if (value > 1023){
        value = 1023;
    }
    CCPR2L = value & 0x00FF;
    CCPR2H = value >> 8; 
}
void SetBlue(int value){
    if (value > 1023){
        value = 1023;
    }
    CCPR3L = value & 0x00FF;
    CCPR3H = value >> 8; 
}
void SetWhite(int value){
    if (value > 1023){
        value = 1023;
    }
    CCPR4L = value & 0x00FF;
    CCPR4H = value >> 8; 
}

void CheckLight(){
    int LichtSterkte, Treshold;
    char state = 0;
    const char Hysteresis = 42 * 2;
    state = 0;
    while (1){
        LichtSterkte = AnaIn(0b010000); //ANC0   
        Treshold     = AnaIn(0b010111); //ANC7
        if (state == 0)    {
            //No detect yet
            if (LichtSterkte > Treshold){
                //Donker
                break;    
            }else{ 
                //Licht
                SetRed(0);
                SetGreen(0);
                SetBlue(0);
                SetWhite(0);
                state = 1;               
            }
        }else{
            //Hysteresis   
            if ((Treshold + Hysteresis) > 1000){
                Treshold = 1000-Hysteresis;
            }
            if (LichtSterkte > (Treshold + Hysteresis)){
                //Donker
                break;                    
            }              
        }
    }
}

void Adcinit(){
    ADCON1bits.ADFM = 1;
    ADCON1bits.ADCS = 0b111;
    ADCON1bits.ADNREF = 0;
    ADCON1bits.ADPREF = 0b00;
    ADCON0bits.ADON = 1;
    
}

int AnaIn(char channel){
    int result;
    
    ADCON0bits.CHS = channel;
    ADCON0bits.GO_nDONE = 1;
    
    while (ADCON0bits.GO_nDONE == 1) {   
    }
 
    result = ADRESH << 8;
    result = result + ADRESL;
    return(result);
}

int LookUp(int value){
    const int dimcurve[] = {
    0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 
    21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 31, 32, 33, 34, 35, 36, 37, 38, 
    39, 40, 41, 42, 43, 44, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 53, 54, 55, 
    56, 57, 58, 59, 60, 61, 62, 62, 63, 64, 65, 66, 67, 68, 69, 69, 70, 71, 72, 
    73, 74, 75, 75, 76, 77, 78, 79, 80, 81, 81, 82, 83, 84, 85, 86, 86, 87, 88,
    89, 90, 91, 91, 92, 93, 94, 95, 96, 96, 97, 98, 99, 100, 100, 101, 102, 103, 
    104, 104, 105, 106, 107, 108, 109, 109, 110, 111, 112, 112, 113, 114, 115, 
    116, 116, 117, 118, 119, 120, 120, 121, 122, 123, 123, 124, 125, 126, 126, 
    127, 128, 129, 130, 130, 131, 132, 133, 133, 134, 135, 136, 136, 137, 138, 
    139, 139, 140, 141, 142, 142, 143, 144, 145, 145, 146, 147, 147, 148, 149, 
    150, 150, 151, 152, 153, 153, 154, 155, 155, 156, 157, 158, 158, 159, 160, 
    160, 161, 162, 163, 163, 164, 165, 165, 166, 167, 167, 168, 169, 170, 170, 
    171, 172, 172, 173, 174, 174, 175, 176, 176, 177, 178, 179, 179, 180, 181, 
    181, 182, 183, 183, 184, 185, 185, 186, 187, 187, 188, 189, 189, 190, 191, 
    191, 192, 193, 193, 194, 195, 195, 196, 197, 197, 198, 198, 199, 200, 200, 
    201, 202, 202, 203, 204, 204, 205, 206, 206, 207, 207, 208, 209, 209, 210, 
    211, 211, 212, 212, 213, 214, 214, 215, 216, 216, 217, 217, 218, 219, 219, 
    220, 221, 221, 222, 222, 223, 224, 224, 225, 225, 226, 227, 227, 228, 228, 
    229, 230, 230, 231, 231, 232, 233, 233, 234, 234, 235, 236, 236, 237, 237, 
    238, 239, 239, 240, 240, 241, 242, 242, 243, 243, 244, 244, 245, 246, 246, 
    247, 247, 248, 248, 249, 250, 250, 251, 251, 252, 252, 253, 254, 254, 255, 
    255, 256, 256, 257, 257, 258, 259, 259, 260, 260, 261, 261, 262, 262, 263, 
    264, 264, 265, 265, 266, 266, 267, 267, 268, 269, 269, 270, 270, 271, 271, 
    272, 272, 273, 273, 274, 274, 275, 276, 276, 277, 277, 278, 278, 279, 279, 
    280, 280, 281, 281, 282, 282, 283, 283, 284, 284, 285, 286, 286, 287, 287, 
    288, 288, 289, 289, 290, 290, 291, 291, 292, 292, 293, 293, 294, 294, 295, 
    295, 296, 296, 297, 297, 298, 298, 299, 299, 300, 300, 301, 301, 302, 302, 
    303, 303, 304, 304, 305, 305, 306, 306, 307, 307, 308, 308, 309, 309, 310, 
    310, 311, 311, 312, 312, 313, 313, 313, 314, 314, 315, 315, 316, 316, 317, 
    317, 318, 318, 319, 319, 320, 320, 321, 321, 322, 322, 322, 323, 323, 324, 
    324, 325, 325, 326, 326, 327, 327, 328, 328, 329, 329, 329, 330, 330, 331, 
    331, 332, 332, 333, 333, 334, 334, 334, 335, 335, 336, 336, 337, 337, 338, 
    338, 338, 339, 339, 340, 340, 341, 341, 342, 342, 342, 343, 343, 344, 344, 
    345, 345, 346, 346, 346, 347, 347, 348, 348, 349, 349, 349, 350, 350, 351, 
    351, 352, 352, 353, 353, 353, 354, 354, 355, 355, 355, 356, 356, 357, 357, 
    358, 358, 358, 359, 359, 360, 360, 361, 361, 361, 362, 362, 363, 363, 363, 
    364, 364, 365, 365, 366, 366, 366, 367, 367, 368, 368, 368, 369, 369, 370, 
    370, 370, 371, 371, 372, 372, 372, 373, 373, 374, 374, 374, 375, 375, 376, 
    376, 376, 377, 377, 378, 378, 378, 379, 379, 380, 380, 380, 381, 381, 382, 
    382, 382, 383, 383, 384, 384, 384, 385, 385, 386, 386, 386, 387, 387, 387, 
    388, 388, 389, 389, 389, 390, 390, 391, 391, 391, 392, 392, 392, 393, 393, 
    394, 394, 394, 395, 395, 395, 396, 396, 397, 397, 397, 398, 398, 398, 399, 
    399, 400, 400, 400, 401, 401, 401, 402, 402, 403, 403, 403, 404, 404, 404, 
    405, 405, 405, 406, 406, 407, 407, 407, 408, 408, 408, 409, 409, 409, 410, 
    410, 410, 411, 411, 412, 412, 412, 413, 413, 413, 414, 414, 414, 415, 415, 
    415, 416, 416, 417, 417, 417, 418, 418, 418, 419, 419, 419, 420, 420, 420, 
    421, 421, 421, 422, 422, 422, 423, 423, 423, 424, 424, 424, 425, 425, 426, 
    426, 426, 427, 427, 427, 428, 428, 428, 429, 429, 429, 430, 430, 430, 431, 
    431, 431, 432, 432, 432, 433, 433, 433, 434, 434, 434, 435, 435, 435, 436, 
    436, 436, 437, 437, 437, 438, 438, 438, 438, 439, 439, 439, 440, 440, 440, 
    441, 441, 441, 442, 442, 442, 443, 443, 443, 444, 444, 444, 445, 445, 445, 
    446, 446, 446, 447, 447, 447, 447, 448, 448, 448, 449, 449, 449, 450, 450, 
    450, 451, 451, 451, 452, 452, 452, 452, 453, 453, 453, 454, 454, 454, 455, 
    455, 455, 456, 456, 456, 457, 457, 457, 457, 458, 458, 458, 459, 459, 459, 
    460, 460, 460, 460, 461, 461, 461, 462, 462, 462, 463, 463, 463, 463, 464, 
    464, 464, 465, 465, 465, 466, 466, 466, 466, 467, 467, 467, 468, 468, 468, 
    468, 469, 469, 469, 470, 470, 470, 471, 471, 471, 471, 472, 472, 472, 473, 
    473, 473, 473, 474, 474, 474, 475, 475, 475, 475, 476, 476, 476, 477, 477, 
    477, 477, 478, 478, 478, 479, 479, 479, 479, 480, 480, 480, 481, 481, 481, 
    481, 482, 482, 482, 482, 483, 483, 483, 484, 484, 484, 484, 485, 485, 485, 
    486, 486, 486, 486, 487, 487, 487, 487, 488, 488, 488, 489, 489, 489, 489, 
    490, 490, 490, 490, 491, 491, 491, 492, 492, 492, 492, 493, 493, 493, 493, 
    494, 494, 494, 494, 495, 495, 495, 496, 496, 496, 496, 497, 497, 497, 497, 
    498, 498, 498, 498, 499, 499, 499, 499, 500, 500, 500, 501, 501, 501, 501, 
    502, 502, 502, 502, 503, 503, 503, 503, 504, 504, 504, 504, 505, 505, 505, 
    505, 506, 506, 506, 506, 507, 507, 507, 507, 508, 508, 508, 508, 509, 509, 
    509, 509, 510, 510, 510, 510, 511, 511
    };
    return(dimcurve[value]);
}
